﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using SQLitePerformance.Resources;
using System.Threading.Tasks;
using Windows.Storage;
using System.IO;
using System.Diagnostics;
using System.Collections.ObjectModel;

namespace SQLitePerformance
{
    public partial class MainPage : PhoneApplicationPage
    {
        SQLiteWinRT.Database db;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

        }

        private async void Button_Start_Click(object sender, RoutedEventArgs e)
        {
            int Route_ID;
            int Service_ID;
            int Trip_ID;
            string Trip_Headsign;
            int Direction_ID;
            int Block_ID;
            int Shape_ID;
            bool Wheelchair_Accessible;

            Button_Start.IsEnabled = false;
            try
            {
                var dbfile = await ApplicationData.Current.LocalFolder.GetFileAsync("sqliteperf.db");
                await dbfile.DeleteAsync();
            }
            catch (FileNotFoundException)
            {
                // This is OK!
            }
            /*********************************************/
            /* Open the Database and create the Schema */
            await LoadDatabase();

            Stopwatch sw = Stopwatch.StartNew();
            int n = 0;

            // Prepare statement
            using (var statement = await db.PrepareStatementAsync("INSERT INTO TTC (id, Route_ID, Service_ID, Trip_ID, Trip_Headsign, Direction_ID, Block_ID, Shape_ID, Wheelchair_Accessible) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)"))
            {
                // Start transaction
                await db.ExecuteStatementAsync("BEGIN TRANSACTION");

                /*********************************************/
                /* Open input file and import into Database*/
                var inputfile = await Windows.ApplicationModel.Package.Current.InstalledLocation.GetFileAsync("trips.txt");
                using (var inputStream = await inputfile.OpenSequentialReadAsync())
                {
                    using (StreamReader rdr = new StreamReader(inputStream.AsStreamForRead()))
                    {
                        while (true)
                        {
                            string inputline = await rdr.ReadLineAsync();
                            if (inputline == null)
                                break;

                            // Discard line 0 - header line
                            if (n > 0)
                            {
                                string[] fields = inputline.Split(new char[] { ',' });

                                Route_ID = Int32.Parse(fields[0]); ;
                                Service_ID = Int32.Parse(fields[1]);
                                Trip_ID = Int32.Parse(fields[2]);
                                Trip_Headsign = fields[3];
                                Direction_ID = Int32.Parse(fields[4]);
                                Block_ID = Int32.Parse(fields[5]);
                                Shape_ID = Int32.Parse(fields[6]);
                                Wheelchair_Accessible = fields[7] == "1" ? true : false;

                                /* Bind parameter values and Insert */
                                statement.Reset();
                                statement.BindIntParameterAt(1, n);
                                statement.BindIntParameterAt(2, Route_ID);
                                statement.BindIntParameterAt(3, Service_ID);
                                statement.BindIntParameterAt(4, Trip_ID);
                                statement.BindTextParameterAt(5, Trip_Headsign);
                                statement.BindIntParameterAt(6, Direction_ID);
                                statement.BindIntParameterAt(7, Block_ID);
                                statement.BindIntParameterAt(8, Shape_ID);
                                statement.BindIntParameterAt(9, Wheelchair_Accessible ? 1 : 0);

                                await statement.StepAsync().AsTask().ConfigureAwait(false);
                            }

                            n++;
                            if (n % 100 == 0)
                            {
                                Dispatcher.BeginInvoke(()=>progressBar.Value = n);
                            }
                        }
                    }
                }

                // Commit transaction
                await db.ExecuteStatementAsync("COMMIT TRANSACTION");
            }

            db.Dispose();
            db = null;

            Dispatcher.BeginInvoke(() =>
                {
                    var report = String.Format(
                        "Inserted {0:d} records in {1:0.00} seconds", 
                        n - 1, (double)sw.ElapsedMilliseconds / 1000);
                    MessageBox.Show(report);

                    Button_Start.IsEnabled = true;
                });
            }

        private async Task LoadDatabase()
        {
            db = new SQLiteWinRT.Database(
                ApplicationData.Current.LocalFolder, "sqliteperf.db");

            await db.OpenAsync();

            // route_id,service_id,trip_id,trip_headsign,direction_id,block_id,shape_id,wheelchair_accessible
            string sql = @"
                CREATE TABLE IF NOT EXISTS TTC 
                    (id INTEGER PRIMARY KEY, 
                     Route_ID INTEGER, 
                     Service_ID INTEGER, 
                     Trip_ID INTEGER, 
                     Trip_Headsign TEXT,
                     Direction_ID INTEGER, 
                     Block_ID INTEGER, 
                     Shape_ID INTEGER, 
                     Wheelchair_Accessible BOOL)";

            await db.ExecuteStatementAsync(sql);
        }


        private async void Button_Select_Click(object sender, RoutedEventArgs e)
        {
            if (db == null)
            {
                db = new SQLiteWinRT.Database(
                    ApplicationData.Current.LocalFolder, "sqliteperf.db");

                await db.OpenAsync();
            }


            Stopwatch sw = Stopwatch.StartNew();
            int n = 0;
            Button_Select.IsEnabled = false;

            string sql = @"
                SELECT
                    id , 
                     Route_ID , 
                     Service_ID, 
                     Trip_ID, 
                     Trip_Headsign,
                     Direction_ID, 
                     Block_ID, 
                     Shape_ID, 
                     Wheelchair_Accessible
                    FROM TTC";
            var stmt = await db.PrepareStatementAsync(sql);

            ObservableCollection<TransportationTrip> tripCollection = 
                new ObservableCollection<TransportationTrip>();

            //while (await stmt.StepAsync())
            while (await stmt.StepAsync().AsTask().ConfigureAwait(false))
            {
                var trip = new TransportationTrip()
                {
                    ID = stmt.GetIntAt(0),
                    RouteID = stmt.GetIntAt(1),
                    ServiceID = stmt.GetIntAt(2),
                    TripID = stmt.GetIntAt(3),
                    TripHeadsign = stmt.GetTextAt(4),
                    DirectionID = stmt.GetIntAt(5),
                    BlockID = stmt.GetIntAt(6),
                    ShapeID = stmt.GetIntAt(7),
                    WheelchairAccessible = stmt.GetIntAt(8) == 1 ? true:false,
                };

                tripCollection.Add(trip);
                n++;

                if (n % 100 == 0)
                {
                    Dispatcher.BeginInvoke(() => progressBar.Value = n);
                }
            }

            db.Dispose();
            db = null;

            Dispatcher.BeginInvoke(() =>
                {
                    var report = String.Format("Selected {0:d} records in {1:0.00} seconds", 
                        n, (double)sw.ElapsedMilliseconds / 1000);
                    MessageBox.Show(report);

                    Button_Select.IsEnabled = true;
                });
        }
    }
}